from ._funcs import (add_paths_to_system, log_to_console)
from ._django import (DjangoMigration, DjangoFixtures, init_django_app, )
from ._constants import *

__all__ = [
	"add_paths_to_system", "log_to_console",
	"init_django_app", "DjangoMigration", "DjangoFixtures",
]



import logging

from msb_auth.users._token_user import TokenUser


def jwt_user_auth_rule(user: TokenUser):
	return user


def jwt_token_settings(**opt):
	from ._constants import JwtAuthConfig as _def
	from datetime import timedelta
	from msb_config import Config

	access_token_validity_in_minutes = Config.get("JWT_ACCESS_TOKEN_VALIDITY").as_int(default=_def.ACCESS_TOKEN_LIFETIME)
	refresh_token_validity_in_minutes = Config.get("JWT_REFRESH_TOKEN_VALIDITY").as_int(default=_def.REFRESH_TOKEN_LIFETIME)

	_jwt_settings = dict()
	try:
		_jwt_settings['TOKEN_USER_CLASS'] = opt.get('token_user_class') or _def.TOKEN_USER_CLASS
		_jwt_settings['USER_ID_CLAIM'] = opt.get('user_id_claim') or _def.USER_ID_CLAIM
		_jwt_settings['USER_AUTHENTICATION_RULE'] = opt.get('user_auth_rule') or _def.USER_AUTHENTICATION_RULE
		_jwt_settings['ALGORITHM'] = opt.get('algorithm') or _def.ALGORITHM
		_jwt_settings['AUTH_HEADER_TYPES'] = opt.get('auth_header_types') or _def.AUTH_HEADER_TYPES
		_jwt_settings['AUTH_HEADER_NAME'] = opt.get('auth_header_name') or _def.AUTH_HEADER_NAME
		_jwt_settings['USER_ID_FIELD'] = opt.get('user_id_field') or _def.USER_ID_FIELD
		_jwt_settings['TOKEN_TYPE_CLAIM'] = opt.get('token_type_claim') or _def.TOKEN_TYPE_CLAIM
		_jwt_settings['JTI_CLAIM'] = opt.get('jti_claim') or _def.JTI_CLAIM
		_jwt_settings['ROTATE_REFRESH_TOKENS'] = opt.get('rotate_refresh_tokens') or _def.ROTATE_REFRESH_TOKENS
		_jwt_settings['BLACKLIST_AFTER_ROTATION'] = opt.get('blacklist_after_rotation') or _def.BLACKLIST_AFTER_ROTATION
		_jwt_settings['AUDIENCE'] = opt.get('audience') or _def.AUDIENCE
		_jwt_settings['ISSUER'] = opt.get('issuer') or _def.ISSUER
		_jwt_settings['SLIDING_TOKEN_REFRESH_EXP_CLAIM'] = opt.get('sliding_token_refresh_exp_claim') or \
		                                                   _def.SLIDING_TOKEN_REFRESH_EXP_CLAIM

		_jwt_settings['ACCESS_TOKEN_LIFETIME'] = timedelta(minutes=access_token_validity_in_minutes)
		_jwt_settings['REFRESH_TOKEN_LIFETIME'] = timedelta(minutes=refresh_token_validity_in_minutes)
		_jwt_settings['SIGNING_KEY'] = Config.get('JWT_TOKEN_SIGNING_KEY').as_str(default=_def.SIGNING_KEY)
		_jwt_settings['VERIFYING_KEY'] = opt.get('verifying_key') or _jwt_settings.get('SIGNING_KEY')

	except Exception as e:
		logging.warning(e)

	return _jwt_settings



import unittest

from ._core import (TestConfig, LiveServerThreadWithReuse)


class PyhonUnitTest(unittest.TestCase):
	port = 8000
	server_thread_class = LiveServerThreadWithReuse

	def setUp(self) -> None:
		"""
		 implement this
		"""
		pass

	@property
	def config(self):
		return TestConfig()



import dataclasses


@dataclasses.dataclass()
class DbVendorConfig:
	query_to_list_tables: str
	query_to_drop_table: str
	excluded_tables: list

	def queries_to_drop_multiple_tables(self, table_list) -> list:
		_drop_table_queries = []
		if isinstance(table_list, list) and len(table_list) > 0:
			for table in table_list:
				if table[0] not in self.excluded_tables and self.query_to_drop_table.strip(" ") != "":
					_drop_table_queries.append(self.query_to_drop_table.format(table_name=table[0]))
		return _drop_table_queries


from ._base import FileInterface, FileBase
from ._exceptions import FileExceptions
from ._generated_file import GeneratedFile
from ._messages import FileMessage

__all__ = [
	"FileInterface", "FileExceptions", "GeneratedFile", "FileMessage", "FileBase"
]


from ._token_user import TokenUser
from ._auth_token import AuthToken
from ._datastructures import (
	UserAccountType, UserPermissions, UserRole, UserName, UserImage,
	UserLocation, UserCountry, UserRegisteredApp
)




def use_django(syspaths: list = None, settings_dir: str = "app", **kwargs):
	syspaths = syspaths if isinstance(syspaths, list) else []

	def outer_func(_func):
		from functools import wraps
		@wraps(_func)
		def inner_func(*args, **kwargs):
			from msb_devtools import init_django_app
			init_django_app(settings_dir=settings_dir, sys_pathlist=syspaths)
			return _func(*args, **kwargs)

		return inner_func

	return outer_func



from dataclasses import dataclass


@dataclass
class SearchParameter:
	_model_filters: dict
	_fields: list = None
	_order_by: str = None
	_limit: int = None
	_offset: int = None
	_order_field: str = None
	_model_filter_map = dict()

	def __init_model_filters(self, filters: dict):
		for key, value in filters.items():
			if key in self._model_filter_map.keys():
				self.add_filters(self._model_filter_map[key], filters[key])

	def __init__(self, **kwargs):
		self._model_filters = dict()
		self._fields = kwargs.get('fields') if kwargs.get('fields') else []
		self._order_by = kwargs.get('order_by') if kwargs.get('order_by') else 'ASC'
		self._limit = kwargs.get('limit') if kwargs.get('limit') else 50
		self._offset = kwargs.get('offset') if kwargs.get('offset') else 0
		self.order_field = kwargs.get('order_field')
		self.__init_model_filters(kwargs.get('filters'))

	def add_filters(self, key: str, value):
		if isinstance(key, str) and value is not None:
			self._model_filters[key] = value

	@property
	def filters(self) -> dict:
		return self._model_filters

	@property
	def fields(self) -> list:
		return self._fields

	@property
	def order_by(self) -> str:
		return self._order_by

	@property
	def limit(self) -> int:
		return self._limit

	@property
	def offset(self) -> int:
		return self._offset

	@property
	def order_field(self) -> str:
		return self._order_field

	@order_field.setter
	def order_field(self, value):
		if value is not None:
			if self.order_by is not 'ASC':
				self._order_field = f"-{value}"
			else:
				self._order_field = value



from rest_framework.permissions import BasePermission


class Permisson(BasePermission):
	def has_permission(self, request, view):
		return True

	def has_object_permission(self, request, view, obj):
		return True


class LoginRequiredPermission(Permisson):

	def has_permission(self, request, view):
		return bool(request.user and request.user.is_authenticated)


class AdminUserPermission(Permisson):
	def has_permission(self, request, view):
		return bool(request.user and request.user.is_staff)



import django
from django.test.testcases import (TestCase,LiveServerThread,QuietWSGIRequestHandler)
from django.core.servers.basehttp import WSGIServer
from msb_dataclasses import Singleton
from os import environ


class ApiTestResult:
	data = None
	message: str = ''
	code: int = 0
	success: bool = False

	def __init__(self, success=None, code=None, message='', data=None,**kwargs):
		self.data = data if data is not None else []
		self.message = message
		self.code = code
		self.success = success

	@property
	def dict(self):
		return self.__dict__


class LiveServerThreadWithReuse(LiveServerThread):
	"""
	This miniclass overrides _create_server to allow port reuse. This avoids creating
	"address already in use" errors for tests that have been run subsequently.
	"""

	def _create_server(self):
		return WSGIServer((self.host, self.port),QuietWSGIRequestHandler,allow_reuse_address=True,)




class TestConfig(metaclass=Singleton):

	def get(self, key: str = None, default=None):
		return environ.get(f"TEST_{str(key).upper()}", default=default)

	@property
	def base_url(self):
		return self.get('BASE_URL', default=environ.get('BASE_URL', default='')).rstrip('/')

	def make_url(self, endpoint: str = ''):
		return f"{self.base_url}/{endpoint.replace('.', '/')}"

	@property
	def default_headers(self):
		_headers = {
			'Content-Type': 'application/json'
		}
		if self.user_agent:
			_headers['HTTP_USER_AGENT'] = self.user_agent

		return _headers

	@property
	def user_agent(self):
		return self.get('USER_AGENT_STRING', default='Test User Agent 1.0')

	@property
	def request_content_type(self):
		return "application/json"

	@property
	def auth_url(self):
		return self.get('AUTH_ENDPOINT', default='')

	@property
	def auth_credentials(self):
		return dict(user=self.get('AUTH_USER'), password=self.get('AUTH_PASSWORD'), auth_type=self.get('AUTH_TYPE'))



from django.utils.functional import cached_property
from .._constants import TokenConst
from msb_cipher import Cipher


class AuthToken:
	token: dict
	_username_field: str = TokenConst.username_field
	_userid_field: str = TokenConst.userid_field
	_user_email_field: str = TokenConst.user_email_field

	def __init__(self, tokendata: dict = None):
		self.token = tokendata if tokendata else dict()

	def __str__(self):
		return f"<{self.__class__.__name__} [{self.id}]: {self.email}>"

	def __hash__(self):
		return hash(self.username)

	def __getattr__(self, attr):
		"""This acts as a backup attribute getter for custom claims defined in Token serializers."""
		return self.token.get(attr, None)

	def set_validation_status(self, status: bool = False):
		self.token['is_valid'] = status

	def _get(self, key: str = '', default=None):
		val = self.payload.get(key)
		return val if val is not None else default

	@cached_property
	def payload(self):
		return self.token.get('payload') or {}

	@property
	def owner(self):
		return Cipher.decrypt(self.token.get('owner'))

	@cached_property
	def id(self):
		return self._get(self._userid_field)

	@cached_property
	def userid(self):
		return self.id

	@cached_property
	def username(self):
		return self._get(self._username_field)

	@property
	def email(self):
		return self._get(self._user_email_field)

	@cached_property
	def auth_type(self):
		return self._get('auth_type')

	@cached_property
	def is_admin(self):
		return self._get("is_admin") == True

	@property
	def is_valid(self):
		return self.token.get('is_valid', False)

	@property
	def is_authenticated(self):
		return self._get('is_authenticated', False) and self.is_valid



from msb_devtools import (init_django_app, DjangoMigration, DjangoFixtures)
from pathlib import Path
import logging,os

APP_DIRECTORY_NAME = "app"
RESOURCE_DIRECTORY_NAME = "resources"
WRITABLE_DIRECTORY_NAME = "writable"
FIXTURES_DIRECTORY_NAME = "fixtures"

BASE_DIR = Path(Path.cwd().__str__().split(RESOURCE_DIRECTORY_NAME)[0])
APP_DIR = BASE_DIR.joinpath(APP_DIRECTORY_NAME)
RESOURCE_DIR = BASE_DIR.joinpath(RESOURCE_DIRECTORY_NAME)
WRITABLE_DIR = BASE_DIR.joinpath(WRITABLE_DIRECTORY_NAME)
FIXTURES_DIR = RESOURCE_DIR.joinpath(FIXTURES_DIRECTORY_NAME)


def get_default_msb_dirs():
	return [BASE_DIR.__str__(), APP_DIR.__str__(), RESOURCE_DIR.__str__(), WRITABLE_DIR.__str__()]


def require_django(_func):
	def inner_func(*args, **kwargs):
		from msb_devtools import init_django_app
		init_django_app(settings_dir=APP_DIRECTORY_NAME, sys_pathlist=get_default_msb_dirs())
		return _func(*args, **kwargs)

	return inner_func


@require_django
def run_database_migrations(app_name: str, dblist: list, is_dev_env: bool, apps: list, **opt):
	try:
		from django.conf import settings
		migration_directory = os.path.join(settings.APP_DIR, app_name, "migrations")

		# perform risks ?
		allow_risky_operation = settings.IS_LOCAL_ENVIRONMENT

		# list of database to run migrations on
		database_list = dblist if len(dblist) > 0 else list(settings.DATABASES.keys())

		drop_tables: bool = opt.get("remove_tables") == True
		remove_files: bool = opt.get("remove_files") == True
		db_to_remove_tables_from: list = opt.get("db_to_remove_tables_from") or []

		migration = DjangoMigration(
			migration_dir=migration_directory,
			is_dev_env=is_dev_env,
			remove_files=remove_files,

		)
		migration.configure(db_list=database_list, apps_list=apps, remove_tables_from_databases=db_to_remove_tables_from)

		return migration.run(remove_tables=drop_tables)
	except Exception as e:
		logging.exception(e)


@require_django
def load_database_fixtures(*from_dirs, is_json=False):
	fixture_extention = "json" if is_json else "yaml"
	for fixture_dir in from_dirs:
		fixture_dir = FIXTURES_DIR.joinpath(fixture_dir.lower())
		fixtures = DjangoFixtures(file_ext=fixture_extention)
		fixtures.load_from_dir(dir=fixture_dir)



from ._singleton import (Singleton)
from ._email import (EmailConfig, EmailConfigData)
from .search_parameter import SearchParameter
from ._wrappers import (ConfigObject)


__all__ = [
	"Singleton", "SearchParameter",
	"EmailConfig", "EmailConfigData", "ConfigObject"
]



from datetime import timedelta

from msb_auth.users._token_user import TokenUser
from msb_config import Config


def jwt_user_auth_rule(user: TokenUser):
	return user


JWT_TOKEN_SETTINGS = {

	'USER_AUTHENTICATION_RULE': 'auth_management.core.rules.jwt_user_auth_rule',
	'TOKEN_USER_CLASS': 'msb_auth.users.TokenUser',
	'ALGORITHM': 'HS256',
	'SIGNING_KEY': Config.get('JWT_TOKEN_SIGNING_KEY', default=None),
	'VERIFYING_KEY': Config.get('JWT_TOKEN_VERIFY_KEY', default=None),

	'AUTH_HEADER_TYPES': ('Bearer',),
	'AUTH_HEADER_NAME': 'HTTP_AUTHORIZATION',
	'USER_ID_FIELD': 'userid',
	'USER_ID_CLAIM': 'id',
	'TOKEN_TYPE_CLAIM': 'token_type',
	'SLIDING_TOKEN_REFRESH_EXP_CLAIM': 'refresh_exp',
	'JTI_CLAIM': 'jti',

	'ACCESS_TOKEN_LIFETIME': timedelta(minutes=int(Config.get('JWT_ACCESS_TOKEN_VALIDITY', default=30).as_str())),
	'REFRESH_TOKEN_LIFETIME': timedelta(minutes=int(Config.get('JWT_REFRESH_TOKEN_VALIDITY', default=1440).as_str())),

	'ROTATE_REFRESH_TOKENS': False,
	'BLACKLIST_AFTER_ROTATION': False,
	'AUDIENCE': Config.get('BASE_URL', default=None),
	'ISSUER': Config.get('BASE_URL', default=None),

}



from ._constants import *
from .results import AuthResult
from .users import (TokenUser)
from .permissions import (LoginRequiredPermission, AdminUserPermission)
from .validators import (JwtTokenValidator)
from ._funcs import (jwt_user_auth_rule, jwt_token_settings)



import os
import sys


def add_paths_to_system(*path_list):
	for path in path_list:
		sys.path.append(path)


def log_to_console(msg, format=False):
	_log_message = f"\n{f'[ {msg} ] ' :*^100}" if format else f"LOG : {msg}"
	return print(_log_message)



from msb_exceptions import ApiException


class CrudApiExceptions:

	class SchemaValidationClassNotDefined(ApiException):
		_message = "Api class inherited from 'ApiViewset' must have a 'validation_schema_class' property defined." \
		           "The value of which should be the class which holds your validation rules."

	class CreateOperationFailed(ApiException):
		_message = "Failed to create the requested data."

	class DeleteOperationFailed(ApiException):
		_message = "Failed to delete the requested data."


	class UpdateOperationFailed(ApiException):
		_message = "Failed to update the requested data."


	class RetrieveOperationFailed(ApiException):
		_message = "Failed to retrieve the requested data."


AUTH_REQUEST_USER_FIELD = "user"
AUTH_REQUEST_PASSWORD_FIELD = "password"

DEFAULT_AUTHENTICATION_CLASSES = ('msb_auth.validators.JwtTokenValidator',)


class TokenConst:
	username_field = "username"
	userid_field = "id"
	user_email_field = "email"


class MsAuthConst:
	AUTH_VALIDATION_USER_VALUE = "token"
	AUTH_VALIDATION_AUTH_TYPE = "ms_sso"


class LdapAuthConst:
	pass


class JwtAuthConfig:
	TOKEN_USER_CLASS: str = 'msb_auth.TokenUser'
	USER_ID_CLAIM: str = 'id'

	USER_AUTHENTICATION_RULE: str = 'msb_auth._defaults.jwt_user_auth_rule'
	ALGORITHM: str = 'HS256'
	AUTH_HEADER_TYPES: str = ('Bearer',)
	AUTH_HEADER_NAME: str = 'HTTP_AUTHORIZATION'
	USER_ID_FIELD: str = 'userid'

	TOKEN_TYPE_CLAIM: str = 'token_type'
	SLIDING_TOKEN_REFRESH_EXP_CLAIM: str = 'refresh_exp'
	JTI_CLAIM: str = 'jti'
	ROTATE_REFRESH_TOKENS: bool = False
	BLACKLIST_AFTER_ROTATION: bool = False

	ACCESS_TOKEN_LIFETIME = 30
	REFRESH_TOKEN_LIFETIME = 1440
	SIGNING_KEY: str = ""
	VERIFYING_KEY: str = SIGNING_KEY
	AUDIENCE: str = None
	ISSUER: str = None



from __future__ import annotations

from dataclasses import dataclass
from typing import Union
from rest_framework import request as drf_request


@dataclass
class RequestWrapper:
	request: Union[drf_request.Request] = None

	@property
	def __meta(self) -> dict:
		return self.request.META or {}

	@property
	def __headers(self) -> dict:
		return self.request.headers or {}

	def get(self, key: str = None, default=None):
		val = self.request.META
		return val if val is not None else default

	@property
	def cookie(self):
		return self.__meta.get('HTTP_COOKIE')

	@property
	def path(self) -> str:
		return self.__meta.get('PATH_INFO')

	@property
	def ip(self):
		return self.__meta.get('REMOTE_ADDR')

	@property
	def method(self) -> str:
		return self.__meta.get('REQUEST_METHOD')

	@property
	def script(self) -> str:
		return self.__meta.get('SCRIPT_NAME')

	@property
	def server(self) -> str:
		return self.__meta.get('SERVER_NAME')

	@property
	def port(self) -> int:
		return int(self.__meta.get('SERVER_PORT'))

	@property
	def protocol(self) -> str:
		return self.__meta.get('SERVER_PROTOCOL')

	@property
	def content_type(self) -> str:
		return self.__meta.get('CONTENT_TYPE')

	@property
	def query_string(self) -> str:
		return self.__meta.get('QUERY_STRING')

	@property
	def authorization(self) -> str:
		return self.__meta.get('HTTP_AUTHORIZATION')

	@property
	def user_agent(self) -> str:
		return self.__headers.get('User-Agent')



import os

from ._constants import (ENVIRONMENT_VARIABLE_NAME, ENV_LOAD_STATUS_KEY, DEBUG_VARIABLE_NAME, DEBUG_VARIABLE_VALUE)
from ._var import EnvVar


class Config:

	@staticmethod
	def load(env_path: str = '', main_file: str = '', local_file: str = ''):
		from ._core import load_config
		load_config(env_path, main_file, local_file)

	@staticmethod
	def is_loaded() -> bool:
		return os.environ.get(ENV_LOAD_STATUS_KEY) is not None

	@staticmethod
	def get(key: str = '', default=None) -> EnvVar:
		return EnvVar(key=key, value=os.environ.get(key, default=default))

	@staticmethod
	def debug():
		return Config.get(DEBUG_VARIABLE_NAME).as_bool(default=DEBUG_VARIABLE_VALUE)

	@staticmethod
	def env_name() -> str:
		return Config.get(ENVIRONMENT_VARIABLE_NAME).as_str(default=None)

	@staticmethod
	def is_local_env():
		return Config.env_name() == 'local'



from pdf2docx import Converter
from .core import FileInterface, FileBase
import os
from pathlib import Path
from ._pdf import PdfFile
from .core import FileExceptions
from .core import GeneratedFile
from .core import FileMessage


class DocxFile(FileBase, FileInterface):
	_extension = None

	def __init__(self):
		self._extension = ".docx"

	def save(self, filename: str, location: str) -> GeneratedFile:
		file = GeneratedFile()
		try:
			content = self.content
			doc_filepath = os.path.join(location, filename+self._extension)
			Path(location).mkdir(parents=True, exist_ok=True)
			if os.path.exists(doc_filepath) and self.file_overwrite is False:
				raise FileExceptions.DocxExistsException
			"""Generate PDF"""
			pdf = PdfFile().set_content(content)
			if self.file_overwrite is True:
				pdf.set_file_overwrite()
			generated_pdf = pdf.save(location=location, filename=filename)
			pdf_path = generated_pdf.path

			if pdf_path != "":
				"""Generate docx file"""
				cv = Converter(pdf_path)
				cv.convert(doc_filepath)
				cv.close()
				if not cv:
					FileExceptions.DocxException()
				else:
					file.path = doc_filepath
					file.message = FileMessage.doc()
					os.remove(pdf_path)
			else:
				raise FileExceptions.PdfException()
		except Exception as e:
			file.message = e
			file.error = True
		finally:
			return file



from dataclasses import dataclass


@dataclass
class UserName:
	first: str
	middle: str
	last: str
	full: str


@dataclass
class UserRole:
	id: int
	name: str


@dataclass
class UserCountry:
	id: int
	name: str


@dataclass
class UserAccountType:
	id: int
	name: str


@dataclass
class UserPermissions:
	pass


@dataclass
class UserImage:
	url: str
	id: int


@dataclass
class UserRegisteredApp:
	id: int
	name: str


@dataclass
class UserLocation:
	id: int
	name: str



from rest_framework_simplejwt.authentication import JWTTokenUserAuthentication

from msb_auth.users import TokenUser
from msb_cipher import Cipher
from msb_http import RequestWrapper


class JwtTokenValidator(JWTTokenUserAuthentication):
	"""
	This class overrides the "JWTTokenUserAuthentication", so that we can implement
	custom authentication rules over the default system

	"""

	_request: RequestWrapper = None

	def __validate_token_owner(self, token: TokenUser = None):
		return f"{self._request.user_agent}@{self._request.ip}" == token.owner

	def __validate_token_integrity(self, token: TokenUser = None) -> bool:
		return token.is_authenticated and self.__validate_token_owner(token=token)

	def authenticate(self, request):
		try:
			self._request = RequestWrapper(request=request)
			auth_result = super(JWTTokenUserAuthentication, self).authenticate(request=request)
			if auth_result:
				token_user: TokenUser = auth_result[0]
				token_user.set_validation_status(self.__validate_token_integrity(token=token_user))
			return auth_result
		except Exception as e:
			return False




class AuthResult:
	auth_type: str = None
	userid: str
	username: str
	email: str

	def __getitem__(self, item):
		return self.auth_data.get(item)

	def __init__(self, auth_username: str, auth_userid: str, **kwargs):
		self.userid = auth_userid
		self.username = auth_username

	@property
	def id(self):
		return self.userid

	@property
	def user(self):
		return self.id

	@property
	def is_authenticated(self) -> bool:
		return (self.username is not None and self.userid is not None)

	@property
	def auth_data(self):
		_auth_data = dict()
		_auth_data['auth_type'] = self.auth_type
		_auth_data['userid'] = self.id
		_auth_data['username'] = self.username
		_auth_data['is_authenticated'] = self.is_authenticated
		return _auth_data



from ._auth_token import AuthToken
from ._datastructures import (
	UserPermissions, UserName, UserRole, UserCountry, UserImage, UserLocation, UserRegisteredApp,
	UserAccountType
)


class TokenUser(AuthToken):

	@property
	def number(self):
		return self._get("employee_number")

	@property
	def name(self) -> UserName:
		return UserName(
			first=self._get("first_name"), middle=self._get("middle_name"),
			last=self._get("last_name"), full=self._get("full_name")
		)

	@property
	def image(self) -> UserImage:
		return self._get("image")



	@property
	def work_id(self):
		return self._get("work_id")

	@property
	def role(self) -> UserRole:
		return UserRole(**self._get('role', default={}))

	@property
	def country(self) -> UserCountry:
		return UserCountry(**self._get('country', default={}))

	@property
	def account_type(self) -> UserAccountType:
		return UserAccountType(id=self._get("account_type_id"), name=self._get("account_type_name"))

	@property
	def permission(self) -> list[UserPermissions]:
		return list[UserPermissions()]

	@property
	def registered_apps(self) -> list[UserRegisteredApp]:
		return [UserRegisteredApp(id=app.get("id"), name=app.get("name")) for app in self._get("registered_apps")]



import os

from cryptography.fernet import Fernet

from msb_exceptions import AppException


class AsymmetricCipher():
	pass


class SymmetricCipher():

	def __init__(self, keyname: str = "SECRET_KEY", key: str = None):
		self.key = os.environ.get('SECRET_KEY', default='')

	def encrypt(self, data=None) -> str:
		if self.key is not None:
			try:
				data = str(data).encode() if not isinstance(data, bytes) else data
				return Fernet(key=self.key).encrypt(data).decode()
			except Exception as e:
				pass

		return data

	def decrypt(self, data=None) -> str:
		if self.key is not None:
			try:
				data = str(data).encode() if not isinstance(data, bytes) else data
				return Fernet(key=self.key).decrypt(data).decode()
			except Exception as e:
				pass
		return data


class Cipher:

	@staticmethod
	def get_instance(asymetric=False):
		if asymetric:
			cipher = AsymmetricCipher()
		else:
			cipher = SymmetricCipher()
		return cipher

	@staticmethod
	def encrypt(data=None, asymetric=False):
		try:
			if data is None:
				raise AppException('Value is None')
			return Cipher.get_instance().encrypt(data)
		except Exception:
			return data

	@staticmethod
	def decrypt(data=None, asymetric=False):
		try:
			if data is None:
				raise AppException('Value is None')
			return Cipher.get_instance().decrypt(data)
		except Exception:
			return data

	@staticmethod
	def decrypt_list_items(*datalist, asymetric=False):
		try:
			return [Cipher.decrypt(i) for i in datalist]
		except Exception as e:
			return datalist

	@staticmethod
	def encrypt_list_items(*datalist, asymetric=False):
		try:
			return [Cipher.encrypt(i) for i in datalist]
		except Exception as e:
			return datalist



from os import environ
from ._endpoint import ApiEndpoint


class ApiHost:
	_app_name: str = 'app'
	_host_url: str = ''
	_auth_token: str = ''
	_endpoint: str = ''

	def __str__(self):
		return f"<{self.__class__.__name__} {self._app_name.upper()} at '{self._host_url}' >"

	def __init__(self, **kwargs):
		self._app_name = kwargs.get('app_name')
		self._host_url = kwargs.get('host_url')
		self._auth_token = kwargs.get('auth_token')

		if self._app_name not in ['', None] and self._host_url in ['', None]:
			self._host_url = environ.get(f'{self._app_name.upper()}_HOST_URL', default='').rstrip('/')

		if self._auth_token not in ['', None] and self._app_name != '':
			self._auth_token = environ.get(f'{self._app_name.upper()}_ACCESS_TOKEN', default='')

	@property
	def host_url(self):
		return f"{self._host_url.rstrip('/')}/{self._app_name}"

	@property
	def endpoint(self):
		return f"/{self._endpoint.rstrip('/').replace('.', '/')}" if self._endpoint else ''

	@property
	def request_url(self):
		return f"{self.host_url}/<api_version>{self.endpoint}"

	def create_endpoint(self, method: str = None, endpoint: str = None, **kwargs) -> ApiEndpoint:
		self._endpoint = endpoint
		return ApiEndpoint(
			url=self.request_url, method=method,
			**{**kwargs, 'auth_token': self._auth_token, 'version_text': "<api_version>"}
		)

	def api_get(self, endpoint: str = None, **kwargs) -> ApiEndpoint:
		return self.create_endpoint(method='GET', endpoint=endpoint, **kwargs)

	def api_put(self, endpoint: str = None, **kwargs) -> ApiEndpoint:
		return self.create_endpoint(method='PUT', endpoint=endpoint, **kwargs)

	def api_post(self, endpoint: str = None, **kwargs) -> ApiEndpoint:
		return self.create_endpoint(method='POST', endpoint=endpoint, **kwargs)

	def api_delete(self, endpoint: str = None, **kwargs) -> ApiEndpoint:
		return self.create_endpoint(method='DELETE', endpoint=endpoint, **kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['msb_apis',
 'msb_apis.services',
 'msb_apis.views',
 'msb_apis.wrappers',
 'msb_auth',
 'msb_auth.permissions',
 'msb_auth.users',
 'msb_auth.validators',
 'msb_cipher',
 'msb_config',
 'msb_dataclasses',
 'msb_datetime',
 'msb_db',
 'msb_db.models',
 'msb_devtools',
 'msb_exceptions',
 'msb_ext',
 'msb_ext.devtools',
 'msb_files',
 'msb_files.core',
 'msb_http',
 'msb_logging',
 'msb_testing',
 'msb_validation']

package_data = \
{'': ['*']}

install_requires = \
['cerberus>=1.3.4,<2.0.0',
 'cffi>=1.15.1,<2.0.0',
 'cryptography>=38.0.3,<39.0.0',
 'django>=4.1.3,<5.0.0',
 'djangorestframework-simplejwt>=5.2.2,<6.0.0',
 'djangorestframework>=3.14.0,<4.0.0',
 'pandas>=1.5.1,<2.0.0',
 'pdf2docx>=0.5.6,<0.6.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'requests>=2.28.1,<3.0.0',
 'xhtml2pdf>=0.2.8,<0.3.0']

setup_kwargs = {
    'name': 'hrin-msb',
    'version': '0.1.6',
    'description': '',
    'long_description': '# hrin-msb\n\n## Pre-requisites for setup\n1. `pip install poetry`\n\n## How To Build\n\n1. `poetry build`\n2. `poetry config http-basic.pypi __token__ <access-token>`\n3. `poetry publish`\n\n\n# Change Log\n ### Version 0.1.1\n***\n\n ### Version 0.1.2\n***\n\n ### Version 0.1.3\n***\n\n1.  Default serializer added to ApiView\n2. fixed incorrect import in _validators.py\n3. fixed msb_database_router\n4. fixed Config.is_local_env() not working\n5. moved devscripts -> devtools\n6. File Utils Added to utils/files\n7. "app_label" removed from "TestConfig" & "ApiTest" Classes\n8. Fixed Bug : \'LoggingModelManager\' object has no attribute \'_queryset_class\'\n9. Fixed : Logging Model not showing any records\n10. Fixed : str method for base model, & removed current_timestamp method from base model\n***\n\n ## Version 0.1.4\n1. Fixed : ModuleNotFoundError: No module named \'pdf2docx\'\n2. Renamed “FileGenerator“ => “FileFactory”,\n3. Add `create_` Prefix in FileFactory methods\n4. Renamed MsbMetaModel -> MsbModelMetaFields\n5. Added validation decorators, and fixed bulk validation issuses\n6. Modified Logging Configuration Files\n7. removed utils package\n8. moved msb_core.wrappers.exceptions -> msb_exceptions\n9. Fixed : Base ApiViews and Crud Routes\n10. Searchparameter class refactored, search method added in ApiService Class\n***\n\n## Version 0.1.41\n1. Fixed: Crud operations not working with encrypted id\'s\n2. Package dependencies updated\n3. Validator test cases refactored\n***\n## Version 0.1.5x\n ### -- Version 0.1.51\n1. dbrouter print statement removed\n2. datetime constants renamed (added _FORMAT to all fof them)\n3. Fixed the default argument in msb_exception  which was causing "DESC :" to log even if desc was none\n4. Api service create methdhod not working correctly\n5. file logging handler is not registered in local env by default, we need to pass `emulate_prod=True` to add it \n6. SearchParameter class imported in package.__init__.py \n7. Fixed : test-cases are breaking because of logs databae\n8. added base unittest class, and modified unit_test to inherit djangoTestCase instead of unittest\n9. Added Validation Schema not defined exceptions\n10. Fixed init_django_app error, int datatype instead of str while setting environement variable.\n11. Added use_django decorator to use over classes/functions\n12. Fixed : MsbMetaModel not working\n13. MsbModel meta fields accessor added\n14. Poetry dependencies updated\n15. DMY_DATE_FORMAT added\n16. versioning changed\n\n  ### -- Version 0.1.52\n1. Fixed : MsbMetaFields not working \n2. Fixed : logging model lot of exceptions are thrown if table applicationlogs is not found\n3. Fixed : logging model exceptions file not found\n4. Fixed : db_migration throwing error if no migration dire is found\n5. renamed use_djnago -> use_django, default value for settinfs_dir is added to "app"\n6. renamend _query in msb_model to db_query\n7. field_type, and label added to configuration model\n8. unique_together constraint added to ConfigurationModel\n9. class DjangoMigration creates migration folder if it doesn\'t exists\n10. Added automatic fixture loading\n11. Fixed : msb_model.__str__() was not able to read the primary key value \n12. comma removed from msbMetamodels\n13. Cipher now supports encryption/decryption of list items \n14. SearchParameter modified to supoprt autmatic filter assignment \n15. Refactor `msb_auth` : TokenUser,AuthResult,Constants added\n16. Jwt Token Validation is strict now, it allows only same owner\n  \n### -- Version 0.1.521\n1. Fixed improper import exception\n\n  \n### -- Version 0.1.522\n1. Added Login Required to base viewset\n2. Added Config Object class to msb_dataclasses \n3. Added msb_http to the package\n4. Added MsbDatabaseRouter in init.py\n5. Fixed model.delete() is not working\n\n### -- Version 0.1.6\n1. Modified django migration script\n2. Aded devtools to msb_ext\n3. removed `use_django` decorator & added `requires_django` decorator\n4. added default values for metafields\n5. added InputFiled.time in validation schema types\n',
    'author': 'Prakash Mishra',
    'author_email': 'prakash.mishra@intimetec.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from ._client import ApiRequestClient
from ._endpoint import ApiEndpoint
from ._host import ApiHost
from ._request import (RequestHeaders, RequestInfo)
from ._response import (ApiResponse,RestResponse)





from msb_exceptions import AppException


class ApiServiceExceptions:
	class InvalidDatabaseModel(AppException):
		_message = "Invalid database model in services."

	class InvalidPk(AppException):
		_message = "Primary Identifier Field cant be empty."

	class InvalidDataForCreateOperation(AppException):
		_message = "Invalid data for creation."

	class CreateOperationFailed(AppException):
		_message = "Create operation request failed."

	class RetrieveOperationFailed(AppException):
		_message = "Failed to retrieve the requested data."

	class ListOperationFailed(AppException):
		_message = "Failed to retrieve the requested list."

	class UpdateOperationFailed(AppException):
		_message = "Update operation failed."

	class BulkUpdateOperationFailed(AppException):
		_message = "Bulk update operation failed."

	class DeleteOperationFailed(AppException):
		_message = "Delete operation failed."

	class BulkDeleteOperationFailed(AppException):
		_message = "Bulk delete operation failed."
